package com.malk.lanyun.controller;

import com.malk.lanyun.service.FService;
import com.malk.lanyun.service.NCService;
import com.malk.lanyun.service.SyncAssetsService;
import com.malk.lanyun.service.TimerService;
import com.malk.server.common.McR;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.util.List;
import java.util.Map;

/**
 * 错误抛出与拦截详见 CatchException
 */
@Slf4j
@RestController
@RequestMapping("/yidaservice")
public class TimerController {

    @Autowired
    private TimerService timerService;

    @Autowired
    private NCService ncService;

    /**
     * 月度定额更新
     */
    @GetMapping("updateYuedudinge")
    public String updateYuedudinge(){

        String updateYuedudinge = timerService.updateYuedudinge();
        return updateYuedudinge;
    }

    @RequestMapping("test")
    McR test() {
        return McR.success();
    }

    @RequestMapping("projectUnshipped")
    McR projectUnshipped() {
        timerService.projectUnshipped();
        return McR.success();
    }

    @RequestMapping("testOne")
    McR testOne() {
        ncService.getNcInfo();
        return McR.success();
    }
    @RequestMapping("testTwo")
    McR testTwo() {
        ncService.getNcCgInfo();
        return McR.success();
    }
    @RequestMapping("testThree")
    McR testThree() {
        ncService.getNcCost();
        return McR.success();
    }
    @RequestMapping("testFour")
    McR testFour() {
        ncService.getNcCollection();
        return McR.success();
    }

    @RequestMapping("testFive")
    McR testFive() {
        ncService.getSupplierPay();
        return McR.success();
    }

    /**
     * 采购订单导出excel
     * @param data 前端传入数据
     * @param response 响应头
     * @param request  请求头
     */
    @PostMapping("exportExecl")
    void exportAttendance(@RequestBody Map data, HttpServletResponse response, HttpServletRequest request) {
        timerService.execlExport(data,response,request);
    }

    /**
     * 分组查询项目点经营分析报表
     *
     * @param data 传参年月
     * @return 返回封装McR
     */
    @PostMapping("projectAnalysisGroup")
    McR projectAnalysisGroup(@RequestBody Map<String,Object> data) {
        Map<String, Object> analysisGroup = timerService.projectAnalysisGroup(data);
        return McR.success(analysisGroup);
    }
    /**
     * 日常物耗申请单中采购专员审批后更新字段初始物耗费用总计
     * @param
     */
    @PostMapping("materialConsumption")
    void updateMaterialConsumption(String formInstId,Double firstCost,Double nowCost){
        log.info("进入更新字段初始物耗费用总计方法");
        timerService.updateMaterialConsumption(formInstId,firstCost,nowCost);
    }

    /**
     * 日常物耗申请单中采购专员提交占用项目点月度物耗定额
     * @param
     */
    @PostMapping("materialProject")
    void updateMaterialProject(String projectCode,String date,Double firstCost,Double nowCost,String formInstId){
        log.info("进入更新字段初始物耗费用总计方法");
        timerService.updateMaterialProject(projectCode,date,firstCost,nowCost,formInstId);
    }

    /**
     * 催款函
     */
    @GetMapping("sendReminderLetter")
    public void updateBigOut() {
        timerService.sendReminderLetter();
    }

    @PostMapping(value = "freshVersion" ,consumes = "application/json")
    public McR freshVersion(@RequestBody Map<String,Object> data){
        log.info(data.toString());
        timerService.freshVersion(data);
        return McR.success();
    }


    /**
     * 项目点档案点编辑保存后，更新是否补充字段为是
     * @param data
     */
    @PostMapping("updateIsSupply")
    public void updateIsSupply(@RequestBody Map data){
        log.info("更新项目点档案是否补充字段");
        timerService.updateIsSupply(data);
    }

    /**
     * 查找项目点档案一个项目编号下所有项目点是否补充
     * @param projectCode
     */
    @PostMapping("selectIsSupply")
    public Map<String, Boolean> selectIsSupply(String projectCode){
        log.info("开始查找项目编号所有项目点是否补充");
        return timerService.selectIsSupply(projectCode);
    }

    @RequestMapping("testSelectIsSupply")
    public Map<String, Boolean> testSelectIsSupply(HttpServletRequest request){
        log.info("开始查找项目编号所有项目点是否补充");
        return timerService.testSelectIsSupply(request);
    }

    /**
     * 单个修改各项目点月度经营分析
     * @param
     * @return
     */
    @PostMapping("updateAnalysis")
    public void updateAnalysis(@RequestBody Map<String,Object> data){
        log.info("开始修改单个各项目点月度经营分析");
        timerService.updateAnalysis(data);
    }

    @PostMapping("checkoutStatus")
    McR checkoutStatus(@RequestBody Map<String,Object> data){
        log.info("开始获取未出库数据：{}",data);
        List<Map> mapList = timerService.checkoutStatus(data);
        return McR.success(mapList);
    }

    /**
     * 日常物耗申请单中发起节点获取首条供应商价目表单价
     * @param
     */
    @PostMapping("updateDailyPrice")
    void updateDailyPrice(String formInstId){
        log.info("进入更新获取首条供应商价目表单价方法");
        timerService.updateDailyPrice(formInstId);
    }

    @Autowired
    private FService fService;

    /**
     * 蓝云, 全量同步账龄表、催款函、各项目点月度经营分析插入数据库
     */
    @PostMapping("syncAgingSchedule")
    public McR syncAgingSchedule(@RequestBody Map data) {
        log.info("传入参数：{}",data);
        String type = data.get("type").toString();
        if ("0".equals(type)){
            log.info("开始同步账龄表");
            fService.syncAgingSchedule();
        } else if ("1".equals(type)) {
            log.info("同步催款函");
            fService.syncCallLetters();
        } else if ("2".equals(type)) {
            log.info("开始调用各项目分析报表插入数据库接口");
            timerService.insertAnalysisList();
        }
        return McR.success();
    }

    @PostMapping("testNC")
    public void testNC(){
        ncService.getProjectOperate();
    }
}
